#pragma once
#include "Deer/DataStore.h"
#include "Deer/Memory.h"
#include "Deer/Path.h"

#ifdef DEER_RENDER
#include "DeerRender/GizmoRenderer.h"
#endif

#include <string>
#include <vector>
#include <array>

namespace Deer {
	class Environment;
	class GizmoRenderer;

	// A scene is a 3d simulation with its environment and voxel world in case
	// of initialized, here things can be simulated
	namespace Scene {
		// Resets all scene to 0 but conserving the memory making it much faster
		// than creating another instance
		void clear();

		// This is the cycle to execution
		void beginExecution();
		void tick();
		void endExecution();

		bool getExecutingState();
#ifdef DEER_RENDER
		// This function renders with the default camera in the environment
		void render();
		void render(SceneCamera);

        extern GizmoRenderer gizmoRenderer;
#endif
		extern Environment environment;
	}

	// Namespace to manage scenes in memory
	namespace DataStore {
		void loadScene(const Path& name);
		void exportScene(const Path& name);

		void exportRuntimeScene();
		void importRuntimeScene();
	}  // namespace DataStore
}  // namespace Deer
